Standard

Struct Standard 

pub struct Standard<O: Options = Default>(/* private fields */);
Defines a standard boundary implementation.

This type is configurable over Options types, but it assumes the Default options when another options type is not provided.

For a description of how boundaries are formed in this implementation, see the boundary module documentation.