SyntaxError

Enum SyntaxError 

pub enum SyntaxError {
    Empty,
    Format(usize),
}
A type for reporting simple string syntax errors.

This type is used within the syntax crate to restrict the number of errors we can report. When validating syntax, it’s usually only one of two things;

  • An empty input, where we expected non-empty
  • An invalid character at a specific byte-offset

Variants



Empty

The value is an empty string, and an empty string is not allowed.



Format(usize)

The format of the string is invalid at the provided byte offset.