Unicode

Enum Unicode 

pub enum Unicode {}
A Unicode Standard Annex #31 profile (with LOW LINE, ZWJ, and ZWNJ removed).

Character Set

This profile uses the unicode-ident crate.

  • Start => any character that passes is_xid_start(c) (except LOW LINE, ZWJ, and ZWNJ).
  • Continue => any character that passes is_xid_continue(c) (except LOW LINE, ZWJ, and ZWNJ).

About Low Line Omission

It’s confusing if the character profile contains a delimiter.

Instead, if you want an identifier that includes the low-line character, you should select such an identifier (from the presets, or by including it as a delimiter in a custom identifier).

We still “include” the character, we just want it to be treated specially.

About ZWJ+ZWNJ Omissions

The zero-width joiner characters (U+200D and U+200C) have special usage within this profile, as detailed in the above specification. However, it requires extra processing to use them properly - we simply omit them.

If you wish to support these extended use-cases, you can create a custom profile which allows these characters (simply by not disallowing them, just call into unicode-ident functions). But you’ll have to implement additional validation on top of the identifiers you form (perhaps as a newtype).

In my reading of this standard, the zero-width characters are just not worth the headache. I really strongly recommend against their inclusion.