LowerCamel

Struct LowerCamel 

pub struct LowerCamel<P>(/* private fields */);
A case adapter for a character profile that accepts uncased and lowercase characters on the first character of a chunk or identifier.

Note

If you’re using the regular unicode profile, it’s possible that you can construct valid strings which appear to break the requirements of this type.

use typed_ident::presets::unicode::LowerCamelIdent;
assert!(LowerCamelIdent::new("_A").is_err()); // Doesn't work (expected).
assert!(LowerCamelIdent::new("_゙A").is_ok()); // Works?

The payload provided for that second case is:

  • _ (U+005F) LOW LINE
  • (U+3099) COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK
  • A (U+0041) LATIN CAPITAL LETTER A

The way to prevent this is to disallow Mn/Mc/Me characters on chunk start.

You can get a profile which does this by enabling unicode-strict. This will enable the Strict profile which can be used directly - or if you have presets enabled - used indirectly through strict presets.

use typed_ident::presets::strict::LowerCamelIdent;
assert!(LowerCamelIdent::new("_A").is_err()); // Doesn't work (expected).
assert!(LowerCamelIdent::new("_゙A").is_err()); // Now it doesn't work!

See Strict for more details on when you should do this.