UpperCamel

Struct UpperCamel 

pub struct UpperCamel<P>(/* private fields */);
A case adapter for a character profile that accepts uncased, uppercase, and titlecase characters on the first character of a chunk or identifier.

Note

If you’re using the regular unicode profile, it’s possible that you can construct valid strings which appear to break the requirements of this type.

use typed_ident::presets::unicode::UpperCamelIdent;
assert!(UpperCamelIdent::new("_a").is_err()); // Doesn't work (expected).
assert!(UpperCamelIdent::new("_゙a").is_ok()); // Works?

The payload provided for that second case is:

  • _ (U+005F) LOW LINE
  • (U+3099) COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK
  • a (U+0061) LATIN SMALL LETTER A

The way to prevent this is to disallow Mn/Mc/Me characters on chunk start.

You can get a profile which does this by enabling unicode-strict. This will enable the Strict profile which can be used directly - or if you have presets enabled - used indirectly through strict presets.

use typed_ident::presets::strict::UpperCamelIdent;
assert!(UpperCamelIdent::new("_a").is_err()); // Doesn't work (expected).
assert!(UpperCamelIdent::new("_゙a").is_err()); // Now it doesn't work!

See Strict for more details on when you should do this.