pub trait Segmentation {
type Graphemes<'a>: GraphemesIterator<'a>;
type GraphemeIndices<'a>: GraphemeIndicesIterator<'a>;
}Expand description
A trait which contains type definitions for segmentation iterators.
Required Associated Types
Sourcetype Graphemes<'a>: GraphemesIterator<'a>
type Graphemes<'a>: GraphemesIterator<'a>
The kind of iterator to use for segmentation where you only need the individual “graphemes” (or grapheme-equivalents).
Sourcetype GraphemeIndices<'a>: GraphemeIndicesIterator<'a>
type GraphemeIndices<'a>: GraphemeIndicesIterator<'a>
Same as Graphemes, except that it additionally will contain byte
offset information ((usize, &str)).
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".