UnitDelimiter

Trait UnitDelimiter 

pub trait UnitDelimiter: Delimiter + Default {
    const CHAR: char;
    const STR: &'static str;
}
A trait that should be implemented when a delimiter has exactly one value, and that value is valid at any position of the identifier.

Required Associated Constants

const CHAR: char

The character value of the delimiter.

const STR: &'static str

A string slice representation of the delimiter.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl UnitDelimiter for HyphenMinus

Source

const CHAR: char = '-'

Source

const STR: &'static str = "-"

Source

impl UnitDelimiter for LowLine

Source

const CHAR: char = '_'

Source

const STR: &'static str = "_"