pub trait ConvertibleIdentifier:
FormattableIdentifier
+ ToLowerCamel
+ ToLowerHybrid
+ ToLowerKebab
+ ToLowerSnake
+ ToUpperCamel
+ ToUpperHybrid
+ ToUpperKebab
+ ToUpperSnake { }Expand description
A marker trait that constrains to only identifiers which support preset
conversion (the To* traits; e.g. ToUpperCamel, ToLowerKebab, etc).
This makes it easier to write generics that you intend on converting.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".