pub trait FormattableIdentifier:
Identifier
+ AsLowerCamel
+ AsLowerHybrid
+ AsLowerKebab
+ AsLowerSnake
+ AsUpperCamel
+ AsUpperHybrid
+ AsUpperKebab
+ AsUpperSnake { }Expand description
A marker trait that constrains to only identifiers which support preset
conversion (the As* traits; e.g. AsUpperCamel, AsLowerKebab, etc).
This makes it easier to write generics that you intend on converting.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".