Module alloc

Module alloc 

A module for defining typed identifier functionality that relies on allocation.

The main things this module provides are:

  • FragmentBuf - The ability to dynamically build a fragment.
  • convert - The ability to convert identifiers from one format to another.
  • Functionality for working with boxed identifiers.

See the respective items for more details.

Re-exports

pub use convert::ConvertibleIdentifier;

Modules

convert
String conversion to another cased format.

Structs

FragmentBuf
A dynamic, growable Fragment.

Type Aliases

StringSegment
An owned, type-erased representation of a Segment.