pub enum Unicode {}Expand description
A Unicode Standard Annex #31 profile (with LOW LINE removed).
Character Set
This profile uses the unicode-ident crate.
- Start => any character that passes
is_xid_start(c). - Continue => any character that passes
is_xid_continue(c)(except LOW LINE).
About Low Line Omission
It’s confusing if the character profile contains a delimiter.
Instead, if you want an identifier that includes the low-line character, you should select such an identifier (from the presets, or by including it as a delimiter in a custom identifier).
We still “include” the character, we just want it to be treated specially.