pub struct Uniform<P>(/* private fields */);Expand description
A case adapter for a character profile that accepts either Lower or
or Upper (but not a mixture of both, like Mixed would).
Caution
Unlike many other profiles, this profile presents an interdependent casing requirement (e.g. the case of the other restricted characters impact the validity of each other).
Because of this, just plainly calling the functions of this Profile may
be more permissive than actually calling the is_ident, is_fragment,
or is_ident_fragment functions.