pub trait UnitDelimiter: Delimiter + Default {
const CHAR: char;
const STR: &'static str;
}Expand description
A trait that should be implemented when a delimiter has exactly one value, and that value is valid at any position of the identifier.
Required Associated Constants
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".